using Test
using RandomCells

println("... for RandomCells:")

@testset "2D Stats" begin
    u = [[1.,0.], [0.,1.], [-1.,0.], [0.,-1.]]
    p = [0.5, 0.5, 0.5, 0.5]
    square = RandomCells.asPolyhedron(u, p)
    a = RandomCells.getVolume(square)
    @test a == 1.0
    l = RandomCells.getSurface(square)
    @test l == 4.0
    n = RandomCells.getFacetNumber(square)
    @test n == 4
end

@testset "3D Stats" begin
    u = [[1.,0.,0.], [0.,1.,0.], [0.,0.,1.], [-1.,0.,0.], [0.,-1.,0.], [0.,0.,-1.]]
    p = [0.5, 0.5, 0.5, 0.5, 0.5, 0.5]
    cube = RandomCells.asPolyhedron(u, p)
    v = RandomCells.getVolume(cube)
    @test v == 1.0
    s = RandomCells.getSurface(cube)
    @test s == 6.0
    b = RandomCells.getMeanWidth(cube)
    @test b == 1.5
    m = RandomCells.getFacetNumber(cube)
    @test m == 6
end

@testset "Random" begin
    P2 = RandomCells.PoissonCell(2)
    @test RandomCells.getFacetNumber(P2) >= 3
    C2 = RandomCells.CroftonCell(2)
    @test RandomCells.getFacetNumber(C2) >= 3
    P3 = RandomCells.PoissonCell(3)
    @test RandomCells.getFacetNumber(P3) >= 4
    C3 = RandomCells.CroftonCell(3)
    @test RandomCells.getFacetNumber(C3) >= 4
    P4 = RandomCells.PoissonCell(4)
    @test isnothing(P4)
    P2neg = RandomCells.PoissonCell(2, -1)
    @test isnothing(P2neg)
    PP2 = RandomCells.getStatsPoissonCell(0)
    @test isnothing(PP2)
    PP2 = RandomCells.getStatsPoissonCell()
    @test length(PP2) == 3
    PP2 = RandomCells.getStatsPoissonCell(4)
    @test length(PP2) == 4
    @test length(PP2[1]) == 3
    PP3 = RandomCells.getStatsPoissonCell(dim=3)
    @test length(PP3) == 4
    PP3 = RandomCells.getStatsPoissonCell(2, dim=3)
    @test length(PP3) == 2
    @test length(PP3[1]) == 4
    PVT2 = RandomCells.PoissonVoronoiCell(2)
    @test RandomCells.getFacetNumber(PVT2) >= 3
    PVZ2 = RandomCells.PoissonVoronoiZeroCell(2)
    @test RandomCells.getFacetNumber(PVZ2) >= 3
    PVT3 = RandomCells.PoissonVoronoiCell(3)
    @test RandomCells.getFacetNumber(PVT3) >= 4
    PVZ3 = RandomCells.PoissonVoronoiZeroCell(3)
    @test RandomCells.getFacetNumber(PVZ3) >= 4
end
